﻿using UnityEngine;
using HarmonyLib;
using System.Collections.Generic;
using System;
using System.Reflection;
using System.Linq;

public class DissociationDissimulation : Mod
{
    public static Dictionary<Network_Player, int> playerValues;
    public static List<(Type, Head[])> heads = new List<(Type, Head[])>
    {
        (typeof(AI_StateMachine_Shark), new Head[] { 127 }),
        (typeof(AI_StateMachine_PufferFish), new Head[] { 206 }),
        (typeof(AI_StateMachine_Boar), new Head[] { 253 }),
        (typeof(AI_StateMachine_MamaBear), new Head[] { 287, 381 }),
        (typeof(AI_StateMachine_Bear), new Head[] { 285, 287, 381 }),
        (typeof(AI_StateMachine_StoneBird), new Head[] { 205, 306 }),
        (typeof(AI_StateMachine_Hyena), new Head[] { 536, 537 }),
        (typeof(AI_StateMachine_HyenaBoss), new Head[] { 537 }),
        (typeof(AI_StateMachine_AnglerFish), new Head[] { 368 }),
        (typeof(AI_StateMachine_Rat), new Head[] { new Head(253,() => ratSpecialRule) }),
        (typeof(AI_StateMachine_ButlerBot), new Head[] { new Head(271, () => butlerSpecialRule), new Head(617, () => butlerSpecialRule) })
    };
    static float detectionRange;
    public static float DetectionRange => ExtraSettingsAPI_Loaded ? detectionRange : 0;
    public static bool ratSpecialRule = false;
    public static bool butlerSpecialRule = false;
    Harmony harmony;
    public void Start()
    {
        playerValues = new Dictionary<Network_Player, int>();
        harmony = new Harmony("com.aidanamite.DissociationDissimulation");
        harmony.PatchAll();
        Log("Mod has been loaded!");
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll(harmony.Id);
        Log("Mod has been unloaded!");
    }
    static bool ExtraSettingsAPI_Loaded = false;

    public void ExtraSettingsAPI_Load() => ExtraSettingsAPI_SettingsClose();
    public void ExtraSettingsAPI_SettingsClose()
    {
        detectionRange = Mathf.Pow(ExtraSettingsAPI_GetSliderValue("detectionRange"), 2);
        ratSpecialRule = ExtraSettingsAPI_GetCheckboxState("ratSpecialRule");
        butlerSpecialRule = ExtraSettingsAPI_GetCheckboxState("butlerSpecialRule");
    }

    public float ExtraSettingsAPI_GetSliderValue(string SettingName) => 0;
    public bool ExtraSettingsAPI_GetCheckboxState(string SettingName) => false;
}

static class ExtentionMethods
{
    public static bool IsWearing(this Network_Player player, Head[] items)
    {
        Equipment[] equips = Traverse.Create(player.PlayerEquipment).Field("equipment").GetValue<Equipment[]>();
        for (int i = 0; i < equips.Length; i++)
            if (equips[i].Equipped && items.Any(x => x.Condition() && x.ItemIndex == equips[i].equipableItem.UniqueIndex))
                    return true;
        return false;
    }
}

[HarmonyPatch(typeof(AI_StateMachine), "UpdateStateMachine")]
public class Patch_ValidTarget
{
    static void Prefix(AI_StateMachine __instance, ref List<Player> __state)
    {
        __state = new List<Player>();
        foreach (Network_Player player in ComponentManager<Raft_Network>.Value.remoteUsers.Values)
            if (!player.PlayerScript.IsDead && (player.FeetPosition - __instance.transform.position).sqrMagnitude > DissociationDissimulation.DetectionRange)
                foreach (var connection in DissociationDissimulation.heads)
                    if (connection.Item1.IsInstanceOfType(__instance) && player.IsWearing(connection.Item2))
                    {
                        player.PlayerScript.IsDead = true;
                        __state.Add(player.PlayerScript);
                    }
    }
    static void Postfix(List<Player> __state)
    {
        foreach (Player player in __state)
            player.IsDead = false;
    }
}

/*[HarmonyPatch(typeof(AI_Component), "Update")]
public class Patch_ValidTarget2
{
    static void Prefix(AI_Component __instance, ref List<Player> __state)
    {
        __state = new List<Player>();
        if (__instance.connectedState?.stateMachine)
            foreach (Network_Player player in ComponentManager<Raft_Network>.Value.remoteUsers.Values)
                if (!player.PlayerScript.IsDead && (player.FeetPosition - __instance.connectedState.stateMachine.transform.position).sqrMagnitude > DissociationDissimulation.DetectionRange)
                    foreach (var connection in DissociationDissimulation.heads)
                        if (connection.Item1.IsInstanceOfType(__instance.connectedState.stateMachine) && player.IsWearing(connection.Item2))
                        {
                            player.PlayerScript.IsDead = true;
                            __state.Add(player.PlayerScript);
                        }
    }
    static void Postfix(List<Player> __state)
    {
        foreach (Player player in __state)
            player.IsDead = false;
    }
}*/

public class Head
{
    public readonly int ItemIndex;
    public readonly Func<bool> Condition;
    public Head(int ItemIndex, Func<bool> Condition = null)
    {
        this.ItemIndex = ItemIndex;
        if (Condition == null)
            Condition = () => true;
        this.Condition = Condition;
    }
    public static implicit operator Head(int v) => new Head(v);
    public static implicit operator int(Head v) => v.ItemIndex;
}